@@ -37,6 +37,8 @@ import binascii
 import string
 import random
 
+from gevent import socket
+
 # Python X2go modules
 import x2go.sshproxy as sshproxy
 import x2go.log as log
@@ -50,6 +52,9 @@ from x2go.backends.info import X2goServerSessionInfo as _X2goServerSessionInfo
 from x2go.backends.info import X2goServerSessionList as _X2goServerSessionList
 from x2go.backends.proxy import X2goProxy as _X2goProxy
 
+from x2go.monkey_patch_paramiko import monkey_patch_paramiko
+monkey_patch_paramiko()
+
 def _rerewrite_blanks(cmd):
     # X2go run command replace X2GO_SPACE_CHAR string with blanks
     if cmd:
@@ -158,10 +163,13 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
     def _x2go_sftp_write(self, remote_path, content):
 
         self.logger('sFTP-write: opening remote file %s on host %s for writing' % (remote_path, self.get_transport().getpeername()), loglevel=log.loglevel_DEBUG)
-        remote_fileobj = self.sftp_client.open(remote_path, 'w')
-        self.logger('sFTP-write: writing content: %s' % content, loglevel=log.loglevel_DEBUG_SFTPXFER)
-        remote_fileobj.write(content)
-        remote_fileobj.close()
+        try:
+            remote_fileobj = self.sftp_client.open(remote_path, 'w')
+            self.logger('sFTP-write: writing content: %s' % content, loglevel=log.loglevel_DEBUG_SFTPXFER)
+            remote_fileobj.write(content)
+            remote_fileobj.close()
+        except SSHException:
+            self.logger('sFTP-write: opening remote file %s on host %s failed' % (remote_path, self.get_transport().getpeername()), loglevel=log.loglevel_WARN)
 
     def _x2go_sftp_remove(self, remote_path):
 
@@ -211,6 +219,12 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
                     self.sshproxy_session.stop_thread()
                 self.locked = False
                 raise x2go_exceptions.X2goControlSessionException('the X2go control session command timed out')
+            except socket.error:
+                self.session_died = True
+                if self.sshproxy_session:
+                    self.sshproxy_session.stop_thread()
+                self.locked = False
+                raise x2go_exceptions.X2goControlSessionException('the X2go control session has died unexpectedly')
             finally:
                 self.locked = False
                 timeout.cancel()
@@ -250,6 +264,11 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
         #return username in self._x2go_remote_group('x2gousers')
         return True
 
+    def is_folder_sharing_available(self):
+        if self.remote_username() in self._x2go_remote_group('fuse'):
+            return True
+        return False
+
     def remote_username(self):
         """\
         Returns the control session's remote username.
@@ -274,6 +293,13 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
         Wraps around a Paramiko/SSH host key check.
 
         """
+        # trailing whitespace tolerance
+        hostname = hostname.strip()
+
+        # force into IPv4 for localhost connections
+        if hostname in ('localhost', 'localhost.localdomain'):
+            hostname = '127.0.0.1'
+
         return checkhosts.check_ssh_host_key(self, hostname, port=port)
 
     def connect(self, hostname, port=22, username='', password='', pkey=None,
@@ -383,13 +409,16 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
             key_filename = None
             pkey = None
 
+        # trailing whitespace tolerance in hostname
+        hostname = hostname.strip()
+
         self.logger('connecting to [%s]:%s' % (hostname, port), loglevel=log.loglevel_NOTICE)
 
         self.load_session_host_keys()
 
         _hostname = hostname
         # enforce IPv4 for localhost address
-        if _hostname == 'localhost':
+        if _hostname in ('localhost', 'localhost.localdomain'):
             _hostname = '127.0.0.1'
 
         if (key_filename and os.path.exists(os.path.normpath(key_filename))) or pkey:
@@ -454,7 +483,7 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
         self.hostname = hostname
         self.port = port
 
-	# if we succeed, we immediately grab us an sFTP client session
+        # if we succeed, we immediately grab us an sFTP client session
         try:
             self.sftp_client = self.open_sftp()
         except:
@@ -470,6 +499,10 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
 
         if self.get_transport():
             self.session_died = False
+
+        if not self.home_exists():
+            raise x2go_exceptions.X2goRemoteHomeException('remote home directory does not exist')
+
         return (self.get_transport() is not None)
 
     def dissociate(self, terminal_session):
@@ -524,6 +557,13 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
             return False
 
 
+    def home_exists(self):
+        (_stdin, _stdout, _stderr) = self._x2go_exec_command('stat -tL "%s"' % self._x2go_remote_home, loglevel=log.loglevel_DEBUG)
+        if _stdout.read():
+            return True
+        return False
+
+
     def is_alive(self):
         if self._x2go_exec_command('echo', loglevel=log.loglevel_DEBUG):
             return True
@@ -725,7 +765,7 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
 
             return _listsessions
 
-    def clean_sessions(self):
+    def clean_sessions(self, destroy_terminals=True):
         """\
         Find X2go terminals that have previously been started by the
         connected user on the remote X2go server and terminate them.
@@ -733,7 +773,7 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
         """
         session_list = self.list_sessions()
         for session_name in session_list.keys():
-            self.terminate(session_name=session_name)
+            self.terminate(session_name=session_name, destroy_terminals=destroy_terminals)
 
     def is_connected(self):
         """\
@@ -790,7 +830,6 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
 
         """
         session_infos = self.list_sessions()
-
         if session_name not in session_infos.keys():
             if session_name in self.terminated_terminals:
                 return True
@@ -799,7 +838,6 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
                 if session_name in self.associated_terminals.keys():
                     self.terminate(session_name)
                 return True
-
         return False
 
     def suspend(self, session_name):
@@ -842,7 +880,7 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
 
         return _ret
 
-    def terminate(self, session_name):
+    def terminate(self, session_name, destroy_terminals=True):
         """\
         Terminate either this or another available X2go session on the connected
         server.
@@ -867,7 +905,7 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
             dummy_stdout = stdout.read()
             dummy_stderr = stderr.read()
             if self.associated_terminals.has_key(session_name):
-                if self.associated_terminals[session_name] is not None:
+                if self.associated_terminals[session_name] is not None and destroy_terminals:
                     self.associated_terminals[session_name].__del__()
                 try: del self.associated_terminals[session_name]
                 except KeyError: pass
