@@ -35,6 +35,7 @@ import threading
 import signal
 import cStringIO
 import copy
+import shutil
 
 # Python X2go modules
 import x2go.rforward as rforward
@@ -325,6 +326,8 @@ class X2goTerminalSessionSTDOUT(object):
         else:
             self.session_info = info_backend()
 
+        self._cleaned_up = False
+
     def __del__(self):
         self._x2go_tidy_up()
 
@@ -363,6 +366,16 @@ class X2goTerminalSessionSTDOUT(object):
             else:
                 raise OSError, e
 
+    def _rm_session_dirtree(self):
+
+        if self.session_info.name:
+            shutil.rmtree('%s/S-%s' % (self.params.rootdir, self.session_info), ignore_errors=True)
+
+    def _rm_desktop_dirtree(self):
+
+        if self.session_info.display:
+            shutil.rmtree('%s/S-%s' % (self.params.rootdir, self.session_info.display), ignore_errors=True)
+
     def get_session_name(self):
         """\
         STILL UNDOCUMENTED
@@ -444,6 +457,9 @@ class X2goTerminalSessionSTDOUT(object):
         Initialize Paramiko/SSH reverse forwarding tunnel for X2go folder sharing.
 
         """
+        if not self.control_session.is_folder_sharing_available():
+            raise x2go_exceptions.X2goUserException('remote user %s is not member of X2go server group fuse' % self.session_info.username)
+
         # start reverse SSH tunnel for sshfs (folder sharing, printing)
         ssh_transport = self.control_session.get_transport()
         if self.reverse_tunnels[self.session_info.name]['sshfs'][1] is None:
@@ -496,7 +512,7 @@ class X2goTerminalSessionSTDOUT(object):
         spool_dir = os.path.join(self.session_info.local_container, 'spool')
         if not os.path.exists(spool_dir):
             os.mkdir(spool_dir)
-        self.share_local_folder(folder_name=spool_dir, folder_type='spool')
+        self.share_local_folder(local_path=spool_dir, folder_type='spool')
         self.print_queue = printqueue.X2goPrintQueue(profile_name=self.profile_name,
                                                      session_name=self.session_info.name,
                                                      spool_dir=spool_dir,
@@ -539,7 +555,7 @@ class X2goTerminalSessionSTDOUT(object):
         mimebox_dir = os.path.join(self.session_info.local_container, 'mimebox')
         if not os.path.exists(mimebox_dir):
             os.mkdir(mimebox_dir)
-        self.share_local_folder(folder_name=mimebox_dir, folder_type='mimebox')
+        self.share_local_folder(local_path=mimebox_dir, folder_type='mimebox')
         self.mimebox_queue = mimebox.X2goMIMEboxQueue(profile_name=self.profile_name,
                                                       session_name=self.session_info.name,
                                                       mimebox_dir=mimebox_dir,
@@ -573,13 +589,13 @@ class X2goTerminalSessionSTDOUT(object):
         """
         return '%s/%s' % (self.session_info.remote_container, 'mimebox')
 
-    def share_local_folder(self, folder_name=None, folder_type='disk'):
+    def share_local_folder(self, local_path=None, folder_type='disk'):
         """\
         Share a local folder with the X2go session.
 
-        @param folder_name: the full path to an existing folder on the local 
+        @param local_path: the full path to an existing folder on the local 
             file system
-        @type folder_name: str
+        @type local_path: str
         @param folder_type: one of 'disk' (a folder on your local hard drive), 'rm' (removeable device), 
             'cdrom' (CD/DVD Rom) or 'spool' (for X2go print spooling)
         @type folder_type: str
@@ -588,23 +604,23 @@ class X2goTerminalSessionSTDOUT(object):
         @rtype: bool
 
         """
-        if self.session_info.username not in self.control_session._x2go_remote_group('fuse'):
+        if not self.control_session.is_folder_sharing_available():
             raise x2go_exceptions.X2goUserException('remote user %s is not member of X2go server group fuse' % self.session_info.username)
 
-        if folder_name is None:
+        if local_path is None:
             self.logger('no folder name given...', log.loglevel_WARN)
             return False
 
-        if type(folder_name) not in (types.StringType, types.UnicodeType):
+        if type(local_path) not in (types.StringType, types.UnicodeType):
             self.logger('folder name needs to be of type StringType...', log.loglevel_WARN)
             return False
 
-        if not os.path.exists(folder_name):
-            self.logger('local folder does not exist: %s' % folder_name, log.loglevel_WARN)
+        if not os.path.exists(local_path):
+            self.logger('local folder does not exist: %s' % local_path, log.loglevel_WARN)
             return False
 
-        folder_name = os.path.normpath(folder_name)
-        self.logger('sharing local folder: %s' % folder_name, log.loglevel_INFO)
+        local_path = os.path.normpath(local_path)
+        self.logger('sharing local folder: %s' % local_path, log.loglevel_INFO)
 
         _auth_rsakey = self.control_session._x2go_session_auth_rsakey
         _host_rsakey = defaults.RSAHostKey
@@ -625,9 +641,9 @@ class X2goTerminalSessionSTDOUT(object):
         _server_encoding = self.params.server_encoding
 
         if _X2GOCLIENT_OS == 'Windows':
-            folder_name = folder_name.replace('\\', '/')
-            folder_name = folder_name.replace(':', '')
-            folder_name = '/windrive/%s' % folder_name
+            local_path = local_path.replace('\\', '/')
+            local_path = local_path.replace(':', '')
+            local_path = '/windrive/%s' % local_path
             _convert_encoding = True
             _client_encoding = 'WINDOWS-1252'
 
@@ -642,9 +658,9 @@ class X2goTerminalSessionSTDOUT(object):
                          'x2gomountdirs', 
                          'dir',
                          str(self.session_info.name), 
-                         _CURRENT_LOCAL_USER,
+                         '"%s"' % _CURRENT_LOCAL_USER,
                          _x2go_key_fname,
-                         '%s__REVERSESSH_PORT__%s; ' % (folder_name, self.session_info.sshfs_port),
+                         '%s__REVERSESSH_PORT__%s; ' % (local_path, self.session_info.sshfs_port),
                          'rm -f %s %s.ident' % (_x2go_key_fname, _x2go_key_fname), 
                        ]
 
@@ -654,9 +670,9 @@ class X2goTerminalSessionSTDOUT(object):
                          'x2gomountdirs', 
                          'dir',
                          str(self.session_info.name), 
-                         _CURRENT_LOCAL_USER,
+                         '"%s"' % _CURRENT_LOCAL_USER,
                          _x2go_key_fname,
-                         '%s__PRINT_SPOOL___REVERSESSH_PORT__%s; ' % (folder_name, self.session_info.sshfs_port),
+                         '%s__PRINT_SPOOL___REVERSESSH_PORT__%s; ' % (local_path, self.session_info.sshfs_port),
                          'rm -f %s %s.ident' % (_x2go_key_fname, _x2go_key_fname), 
                        ]
 
@@ -666,14 +682,65 @@ class X2goTerminalSessionSTDOUT(object):
                          'x2gomountdirs', 
                          'dir',
                          str(self.session_info.name), 
-                         _CURRENT_LOCAL_USER,
+                         '"%s"' % _CURRENT_LOCAL_USER,
                          _x2go_key_fname,
-                         '%s__MIMEBOX_SPOOL___REVERSESSH_PORT__%s; ' % (folder_name, self.session_info.sshfs_port),
+                         '%s__MIMEBOX_SPOOL___REVERSESSH_PORT__%s; ' % (local_path, self.session_info.sshfs_port),
                          'rm -f %s %s.ident' % (_x2go_key_fname, _x2go_key_fname), 
                        ]
 
         (stdin, stdout, stderr) = self.control_session._x2go_exec_command(cmd_line)
-        self.logger('x2gomountdirs output is : %s' % stdout.read().split('\n'), log.loglevel_NOTICE)
+        _stdout = stdout.read().split('\n')
+        self.logger('x2gomountdirs output is : %s' % _stdout, log.loglevel_NOTICE)
+        if _stdout[5].endswith('ok'):
+            return True
+        return False
+
+    def unshare_all_local_folders(self):
+        """\
+        Unshare all local folders mount in the X2go session.
+
+        @return: returns C{True} if all local folders could be successfully unmounted from the X2go server session
+        @rtype: bool
+
+        """
+        self.logger('unsharing all local folders from session %s' % self.session_info, log.loglevel_INFO)
+
+        cmd_line = [ 'export HOSTNAME &&',
+                     'x2goumount-session', 
+                     self.session_info.name,
+                   ]
+
+        (stdin, stdout, stderr) = self.control_session._x2go_exec_command(cmd_line)
+        if not stderr.read():
+            self.logger('x2goumount-session (all mounts) for session %s has been successful' % self.session_info, log.loglevel_NOTICE)
+            return True
+        else:
+            self.logger('x2goumount-session (all mounts) for session %s failed' % self.session_info, log.loglevel_ERROR)
+            return False
+
+    def unshare_local_folder(self, local_path):
+        """\
+        Unshare local folder given as <local_path> from X2go session.
+
+        @return: returns C{True} if the local folder <local_path> could be successfully unmounted from the X2go server session
+        @rtype: bool
+
+        """
+        self.logger('unsharing local folder from session %s' % self.session_info, log.loglevel_INFO)
+
+        cmd_line = [ 'export HOSTNAME &&',
+                     'x2goumount-session', 
+                     self.session_info.name,
+                     local_path,
+                   ]
+
+        (stdin, stdout, stderr) = self.control_session._x2go_exec_command(cmd_line)
+        if not stderr.read():
+            self.logger('x2goumount-session (%s) for session %s has been successful' % (local_path, self.session_info, ), log.loglevel_NOTICE)
+            return True
+        else:
+            self.logger('x2goumount-session (%s) for session %s failed' % (local_path, self.session_info, ), log.loglevel_ERROR)
+            return False
 
     def color_depth(self):
         """\
@@ -690,6 +757,8 @@ class X2goTerminalSessionSTDOUT(object):
         Verify if the command <cmd> exists on the X2go server.
 
         """
+        test_cmd = None;
+
         cmd = cmd.strip('"').strip('"')
         if cmd.find('RDP') != -1:
             cmd = 'rdesktop'
@@ -698,7 +767,11 @@ class X2goTerminalSessionSTDOUT(object):
             return True
         elif 'XSHAD' in cmd:
             return True
-        elif cmd:
+        elif cmd and cmd.startswith('/'):
+            # check if full path is correct _and_ if application is in server path
+            test_cmd = 'test -x %s && which %s && echo OK' % (cmd, os.path.basename(cmd.split()[0]))
+        elif cmd and '/' not in cmd.split()[0]:
+            # check if application is in server path only
             test_cmd = 'which %s && echo OK' % os.path.basename(cmd.split()[0])
 
         if test_cmd:
@@ -745,15 +818,19 @@ class X2goTerminalSessionSTDOUT(object):
 
         self.params.update({'cmd': cmd})
 
+        # do not allow the execution of full path names
+        if '/' in cmd:
+            cmd = os.path.basename(cmd)
+
         cmd_line = [ "setsid x2goruncommand", 
                      str(self.session_info.display),
                      str(self.session_info.agent_pid),
                      str(self.session_info.name), 
                      str(self.session_info.snd_port),
-                     _rewrite_blanks(_rewrite_cmd(self.params.cmd, params=self.params)),
+                     _rewrite_blanks(_rewrite_cmd(cmd, params=self.params)),
                      str(self.params.snd_system),
                      str(self.params.session_type),
-                     ">& /dev/null & exit",
+                     "&> /dev/null & exit",
                    ]
 
         if self.params.snd_system == 'pulse':
@@ -827,6 +904,10 @@ class X2goTerminalSessionSTDOUT(object):
         if self.params.kblayout or self.params.kbtype:
             setkbd = "1"
 
+        cmd = self.params.cmd
+        if '/' in cmd:
+            cmd = os.path.basename(cmd)
+
         cmd_line = [ "x2gostartagent",
                      str(self.params.geometry),
                      str(self.params.link),
@@ -836,7 +917,7 @@ class X2goTerminalSessionSTDOUT(object):
                      str(self.params.kbtype),
                      str(setkbd),
                      str(self.params.session_type),
-                     self.params.cmd,
+                     cmd,
                    ]
 
         if self.params.cmd == 'XDMCP' and self.params.xdmcp_server:
@@ -852,10 +933,15 @@ class X2goTerminalSessionSTDOUT(object):
         if "ACCESS DENIED" in _stderr and "XSHAD" in _stderr:
             raise x2go_exceptions.X2goDesktopSharingException('X2go desktop sharing has been denied by the remote user')
 
-        self.session_info.initialize(_stdout,
-                                     username=self.control_session.remote_username(),
-                                     hostname=self.control_session.get_transport().getpeername(),
-                                    )
+        try:
+            self.session_info.initialize(_stdout,
+                                         username=self.control_session.remote_username(),
+                                         hostname=self.control_session.get_transport().getpeername(),
+                                        )
+        except ValueError:
+            raise X2goTerminalSessionException("failed to start X2go session")
+        except IndexError:
+            raise X2goTerminalSessionException("failed to start X2go session")
 
         # local path may be a Windows path, so we use the path separator of the local system
         self.session_info.local_container = os.path.join(self.params.rootdir, 'S-%s' % self.session_info.name)
@@ -949,8 +1035,10 @@ class X2goTerminalSessionSTDOUT(object):
         @rtype: bool
 
         """
-        self.control_session.terminate(session_name=self.session_info.name)
+        self.control_session.terminate(session_name=self.session_info.name, destroy_terminals=False)
         self.release_proxy()
+        self.post_terminate_cleanup()
+        self.__del__()
 
         # TODO: check if session has really suspended
         _ret = True
@@ -965,3 +1053,23 @@ class X2goTerminalSessionSTDOUT(object):
         if self.proxy is not None:
             self.proxy.__del__()
 
+    def post_terminate_cleanup(self):
+        """\
+        STILL UNDOCUMENTED
+
+        """
+        # this method might be called twice (directly and from update_status in the session
+        # registry instance. So we have to make sure, that this code will not fail
+        # if called twice.
+        if not self._cleaned_up and self.session_info.name:
+
+            # otherwise we wipe the session files locally
+            self.logger('cleaning up session %s after termination' % self.session_info, loglevel=log.loglevel_NOTICE)
+
+            # if we run in debug mode, we keep local session directories
+            if self.logger.get_loglevel() & log.loglevel_DEBUG != log.loglevel_DEBUG:
+
+                self._rm_session_dirtree()
+                self._rm_desktop_dirtree()
+
+            self._cleaned_up = True
