@@ -26,6 +26,7 @@ __NAME__ = 'x2gocheckhosts-pylib'
 # modules
 import paramiko
 import binascii
+import uuid
 
 # Python X2go modules
 import sshproxy
@@ -96,7 +97,7 @@ class X2goInteractiveAddPolicy(paramiko.MissingHostKeyPolicy):
                                                                   fingerprint_type=self.get_key_name(),
                                                                  )
             if _valid:
-                paramiko.AutoAddPolicy().missing_host_key(client, hostname, key)
+                paramiko.AutoAddPolicy().missing_host_key(client, self.hostname, key)
             else:
                 if type(self.caller) in (sshproxy.X2goSSHProxy, ):
                     raise x2go_exceptions.X2goSSHProxyHostKeyException('Invalid host %s is not authorized for access. Add the host to Paramiko/SSH\'s known_hosts file.' % hostname)
@@ -221,7 +222,7 @@ def check_ssh_host_key(x2go_sshclient_instance, hostname, port=22):
 
     host_ok = False
     try:
-        paramiko.SSHClient.connect(x2go_sshclient_instance, hostname=hostname, port=port, username='foo', password='bar')
+        paramiko.SSHClient.connect(x2go_sshclient_instance, hostname=hostname, port=port, username='foo', password="".join([random.choice(string.letters+string.digits) for x in range(1, 20)]))
     except x2go_exceptions.AuthenticationException:
         host_ok = True
         x2go_sshclient_instance.logger('SSH host key verification for host [%s]:%s succeeded. Host is already known to the client\'s Paramiko/SSH sub-system.' % (_hostname, _port), loglevel=log.loglevel_NOTICE)
