@@ -156,9 +156,10 @@ class X2goIniFile(object):
         """
         if type(value) == type(u''):
             value = value.encode(utils.get_encoding())
-
         if type(value) is types.BooleanType:
             self.iniConfig.set(section, key, str(int(value)))
+        elif type(value) in (types.ListType, types.TupleType):
+            self.iniConfig.set(section, key, ", ".join(value))
         else:
             self.iniConfig.set(section, key, str(value))
 
@@ -249,8 +250,10 @@ class X2goIniFile(object):
                 val = self.iniConfig.get(section, key)
                 if val.startswith('[') and val.endswith(']'):
                     return eval(val)
+                elif ',' in val:
+                    val = [ v.strip() for v in val.split(',') ]
                 else:
-                    raise TypeError
+                    val = [ val ]
             else:
                 _val = self.iniConfig.get(section, key)
                 return _val.decode(utils.get_encoding())
