@@ -236,6 +236,7 @@ class X2goSession(object):
         self.terminal_params = {}
         self.sshproxy_params = {}
         self.update_params(params)
+        self.shared_folders = []
 
         self.session_environment = {}
 
@@ -270,9 +271,24 @@ class X2goSession(object):
             'faulty': self.faulty,
         }
 
+        self._SUPPORTED_SOUND = SUPPORTED_SOUND
+        self._SUPPORTED_PRINTING = SUPPORTED_PRINTING
+        self._SUPPORTED_MIMEBOX = SUPPORTED_MIMEBOX
+        self._SUPPORTED_FOLDERSHARING = SUPPORTED_FOLDERSHARING
+
         self.init_control_session()
         self.terminal_session = None
 
+    def HOOK_session_startup_failed(self):
+        """\
+        HOOK method: called if the startup of a session failed.
+
+        """
+        if self.client_instance:
+            self.client_instance.HOOK_session_startup_failed(profile_name=self.profile_name)
+        else:
+            self.logger('HOOK_session_startup_failed: session startup for session profile ,,%s'' failed.' % self.profile_name, loglevel=log.loglevel_WARN)
+
     def HOOK_rforward_request_denied(self, server_port=0):
         """\
         HOOK method: called if a reverse port forwarding request has been denied.
@@ -665,6 +681,37 @@ class X2goSession(object):
         return _valid or self.HOOK_check_host_dialog(host=_host, port=_port, fingerprint=_fingerprint, fingerprint_type=_fingerprint_type)
     __check_host = check_host
 
+    def uses_sshproxy(self):
+        """\
+        Check if a session is configured to use an intermediate SSH proxy server.
+
+        @return: returns C{True} if the session is configured to use an SSH proxy, C{False} otherwise.
+        @rtype: C{bool}
+
+        """
+        return self.use_sshproxy
+
+    def can_sshproxy_auto_connect(self):
+        """\
+        Check if a session's SSH proxy (if used) is configured adequately to be able to auto-connect
+        to the SSH proxy server (e.g. by public key authentication).
+
+        @return: returns C{True} if the session's SSH proxy can auto-connect, C{False} otherwise, C{None}
+            if no SSH proxy is used for this session, C{None} is returned.
+        @rtype: C{bool}
+
+        """
+        if self.use_sshproxy:
+            if self.sshproxy_params.has_key('sshproxy_key_filename') and self.sshproxy_params['sshproxy_key_filename'] and os.path.exists(os.path.normpath(self.sshproxy_params['sshproxy_key_filename'])):
+                return True
+            elif self.sshproxy_params.has_key('sshproxy_pkey') and self.sshproxy_params['sshproxy_pkey']:
+                return True
+            else:
+                return False
+        else:
+            return None
+    __can_sshproxy_auto_connect = can_sshproxy_auto_connect
+
     def can_auto_connect(self):
         """\
         Check if a session is configured adequately to be able to auto-connect to the X2go
@@ -675,26 +722,24 @@ class X2goSession(object):
         @rtype: C{bool}
 
         """
-
-        def _can_sshproxy_autoconnect():
-
-            if self.use_sshproxy:
-                if self.sshproxy_params.has_key('sshproxy_key_filename') and self.sshproxy_params['sshproxy_key_filename'] and os.path.exists(os.path.normpath(self.sshproxy_params['sshproxy_key_filename'])):
-                    return True
-                elif self.sshproxy_params.has_key('sshproxy_pkey') and self.sshproxy_params['sshproxy_pkey']:
-                    return True
-                else:
-                    return False
-            else:
-                return True
+        if self.control_session is None:
+            return None
 
         # do we have a key file passed as control parameter?
         if self.control_params.has_key('key_filename') and self.control_params['key_filename'] and os.path.exists(os.path.normpath(self.control_params['key_filename'])):
-            return _can_sshproxy_autoconnect()
+            _can_sshproxy_auto_connect = self.can_sshproxy_auto_connect()
+            if _can_sshproxy_auto_connect is not None:
+                return _can_sshproxy_auto_connect
+            else:
+                return True
 
         # or a private key?
         elif self.control_params.has_key('pkey') and self.control_params['pkey']:
-            return _can_sshproxy_autoconnect()
+            _can_sshproxy_auto_connect = self.can_sshproxy_auto_connect()
+            if _can_sshproxy_auto_connect is not None:
+                return _can_sshproxy_auto_connect
+            else:
+                return True
 
         else:
             return False
@@ -757,6 +802,9 @@ class X2goSession(object):
                                                               **_params)
             except X2goControlSessionException, e:
                 raise X2goSessionException(str(e))
+            except X2goRemoteHomeException, e:
+                self.disconnect()
+                raise e
             except:
                 # remove credentials immediately
                 self.control_params['password'] = ''
@@ -837,13 +885,13 @@ class X2goSession(object):
         return self.connected
     __is_alive = is_alive
 
-    def clean_sessions(self):
+    def clean_sessions(self, destroy_terminals=True):
         """\
         Clean all running sessions for the authenticated user on the remote X2go server.
 
         """
         if self.is_alive():
-            self.control_session.clean_sessions()
+            self.control_session.clean_sessions(destroy_terminals=destroy_terminals)
         else:
             self._X2goSession__disconnect()
     __clean_sessions = clean_sessions
@@ -863,7 +911,7 @@ class X2goSession(object):
         try:
             return self.control_session.list_sessions(raw=raw)
         except X2goControlSessionException:
-            self._X2goSession_disconnect()
+            self._X2goSession__disconnect()
             return None
     __list_sessions = list_sessions
 
@@ -887,7 +935,7 @@ class X2goSession(object):
             else:
                 return []
         except X2goControlSessionException:
-            self._X2goSession_disconnect()
+            self._X2goSession__disconnect()
             return None
     __list_desktops = list_desktops
 
@@ -917,7 +965,7 @@ class X2goSession(object):
 
             # skip this session status update if not longer than a second ago...
             if  _status_update_timedelta < 1:
-                self.logger('status update interval too short (%s), skipping status about this time...' % _status_update_timedelta, loglevel=log.loglevel_DEBUG)
+                self.logger('status update interval too short (%s), skipping status update this time...' % _status_update_timedelta, loglevel=log.loglevel_DEBUG)
                 return False
 
         e = None
@@ -1005,38 +1053,50 @@ class X2goSession(object):
                 try:
                     self.session_name = self.terminal_session.session_info.name
                 except AttributeError:
-                    raise X2goSessionException('start of new X2go session failed')
+                    self.HOOK_session_startup_failed()
+                    return False
 
             if self.has_terminal_session() and not self.faulty:
 
-                if SUPPORTED_SOUND and self.terminal_session.params.snd_system is not 'none':
-                    self.terminal_session and not self.faulty and self.terminal_session.start_sound()
+                # only run the session startup command if we do not resume...
+                if _new_session:
+                    self.terminal_session.run_command(env=self.session_environment)
 
-                if (SUPPORTED_PRINTING and self.printing) or \
-                   (SUPPORTED_MIMEBOX and self.allow_mimebox) or \
-                   (SUPPORTED_FOLDERSHARING and self.allow_share_local_folders):
-                    self.terminal_session and not self.faulty and self.terminal_session.start_sshfs()
+                if self._SUPPORTED_SOUND and self.terminal_session.params.snd_system is not 'none':
+                    self.terminal_session and not self.faulty and self.terminal_session.start_sound()
+                else:
+                    self._SUPPORTED_SOUND = False
+
+                try:
+                    if (self._SUPPORTED_PRINTING and self.printing) or \
+                       (self._SUPPORTED_MIMEBOX and self.allow_mimebox) or \
+                       (self._SUPPORTED_FOLDERSHARING and self.allow_share_local_folders):
+                        self.terminal_session and not self.faulty and self.terminal_session.start_sshfs()
+                except X2goUserException, e:
+                    self.logger('%s' % str(e), loglevel=log.loglevel_WARN)
+                    # TODO: handle this exception as a notification hook method...
+                    self._SUPPORTED_PRINTING = False
+                    self._SUPPORTED_MIMEBOX = False
+                    self._SUPPORTED_FOLDERSHARING = False
 
                 try:
                     if SUPPORTED_PRINTING and self.printing:
                         self.terminal_session and not self.faulty and self.terminal_session.start_printing()
                         self.terminal_session and not self.faulty and self.session_environment.update({'X2GO_SPOOLDIR': self.terminal_session.get_printing_spooldir(), })
-                except X2goUserException:
-                    pass
+                except X2goUserException, e:
+                    self.logger('%s' % str(e), loglevel=log.loglevel_WARN)
+                    # TODO: handle this exception as a notification hook method...
+                    self._SUPPORTED_PRINTING = False
 
-                if SUPPORTED_MIMEBOX and self.allow_mimebox:
+                if self._SUPPORTED_MIMEBOX and self.allow_mimebox:
                     self.terminal_session and not self.faulty and self.terminal_session.start_mimebox(mimebox_extensions=self.mimebox_extensions, mimebox_action=self.mimebox_action)
-                    self.session_environment.update({'X2GO_MIMEBOX': self.terminal_session.get_mimebox_spooldir(), })
+                    self.terminal_session and self.session_environment.update({'X2GO_MIMEBOX': self.terminal_session.get_mimebox_spooldir(), })
 
-                if SUPPORTED_FOLDERSHARING and self.share_local_folders:
+                if self.share_local_folders and self.terminal_session and not self.faulty and self.is_folder_sharing_available():
                     if _control.get_transport().reverse_tunnels[self.terminal_session.get_session_name()]['sshfs'][1] is not None:
                         for _folder in self.share_local_folders:
                             self.share_local_folder(_folder)
 
-                # only run the session startup command if we do not resume...
-                if _new_session:
-                    self.terminal_session.run_command(env=self.session_environment)
-
                 self.virgin = False
                 self.suspended = False
                 self.running = True
@@ -1134,11 +1194,11 @@ class X2goSession(object):
                 return self.running
             else:
                 self.terminal_session = None
-                return False
 
         else:
             self._X2goSession__disconnect()
-            return False
+
+        return False
     __share_desktop = share_desktop
 
     def suspend(self):
@@ -1151,6 +1211,7 @@ class X2goSession(object):
         """
         if self.is_alive():
             if self.has_terminal_session():
+
                 if self.terminal_session.suspend():
 
                     self.running = False
@@ -1189,8 +1250,8 @@ class X2goSession(object):
         """
         if self.is_alive():
             if self.has_terminal_session():
-                if self.terminal_session.terminate():
 
+                if self.terminal_session.terminate():
                     self.running = False
                     self.suspended = False
                     self.terminated = True
@@ -1350,15 +1411,31 @@ class X2goSession(object):
                 self.faulty = False
             if self.virgin and not self.terminated:
                 self.terminated = None
-        return self.has_terminated
+        return self.terminated
     __has_terminated = has_terminated
 
-    def share_local_folder(self, folder_name):
+    def is_folder_sharing_available(self):
+        """\
+        Test if the remote session allows sharing of local folders with the session.
+
+        @return: returns C{True} if local folder sharing is available in the remote session
+        @rtype: C{bool}
+
+        """
+        if self._SUPPORTED_FOLDERSHARING and self.allow_share_local_folders:
+            if self.is_connected():
+                return self.control_session.is_folder_sharing_available()
+        else:
+            self.logger('local folder sharing is disabled for this session profile', loglevel=log.loglevel_WARN)
+
+    def share_local_folder(self, local_path=None, folder_name=None):
         """\
         Share a local folder with this registered X2go session.
 
-        @param folder_name: the full path to an existing folder on the local
+        @param local_path: the full path to an existing folder on the local
             file system
+        @type local_path: C{str}
+        @param folder_name: synonymous to C{local_path}
         @type folder_name: C{str}
 
         @return: returns C{True} if the local folder has been successfully mounted within
@@ -1366,15 +1443,86 @@ class X2goSession(object):
         @rtype: C{bool}
 
         """
+        # compat for Python-X2go (<=0.1.1.6)
+        if folder_name: local_path=folder_name
+
         if self.has_terminal_session():
-            if self.allow_share_local_folders:
-                return self.terminal_session.share_local_folder(folder_name=folder_name)
+            if self._SUPPORTED_FOLDERSHARING and self.allow_share_local_folders:
+                if self.terminal_session.share_local_folder(local_path=local_path):
+                    self.shared_folders.append(local_path)
+                    return True
+                return False
             else:
                 self.logger('local folder sharing is disabled for this session profile', loglevel=log.loglevel_WARN)
         else:
             raise X2goSessionException('this X2goSession object does not have any associated terminal')
     __share_local_folder = share_local_folder
 
+    def unshare_all_local_folders(self, force_all=False):
+        """\
+        Unshare all local folders mounted within this X2go session.
+
+        @param force_all: Really unmount _all_ shared folders, including the print spool folder and
+            the MIME box spool dir (not recommended).
+        @type force_all: C{bool}
+
+        @return: returns C{True} if all local folders could be successfully unmounted
+            inside this X2go session
+        @rtype: C{bool}
+
+        """
+        if self.has_terminal_session():
+            if self._SUPPORTED_FOLDERSHARING and self.allow_share_local_folders:
+                if force_all:
+                    self.shared_folders = []
+                    return self.terminal_session.unshare_all_local_folders()
+                else:
+                    retval = 0
+                    for _shared_folder in self.shared_folders:
+                        retval = retval | self.terminal_session.unshare_local_folder(_shared_folder)
+                    self.shared_folders = []
+                    return retval
+            else:
+                self.logger('local folder sharing is disabled for this session profile', loglevel=log.loglevel_WARN)
+        else:
+            raise X2goSessionException('this X2goSession object does not have any associated terminal')
+        return False
+    __unshare_all_local_folders = unshare_all_local_folders
+
+    def unshare_local_folder(self, local_path=None):
+        """\
+        Unshare a local folder that is mounted within this X2go session.
+
+        @param local_path: the full path to an existing folder on the local
+            file system that is mounted in this X2go session and shall be
+            unmounted
+        @type local_path: C{str}
+        @return: returns C{True} if all local folders could be successfully unmounted
+            inside this X2go session
+        @rtype: C{bool}
+
+        """
+        if self.has_terminal_session():
+            if self._SUPPORTED_FOLDERSHARING and self.allow_share_local_folders and local_path in self.shared_folders:
+                self.shared_folders.remove(local_path)
+                return self.terminal_session.unshare_local_folder(local_path=local_path)
+            else:
+                self.logger('local folder sharing is disabled for this session profile', loglevel=log.loglevel_WARN)
+        else:
+            raise X2goSessionException('this X2goSession object does not have any associated terminal')
+    __unshare_local_folder = unshare_local_folder
+
+    def get_shared_folders(self):
+        """\
+        Get a list of local folders mounted within this X2go session from this client.
+
+        @return: returns a C{list} of those folder names that are mounted with this X2go session.
+        @rtype: C{list}
+
+        """
+        return self.shared_folders
+    __get_shared_folders = get_shared_folders
+
     def is_locked(self):
         """\
         Query session if it is locked by some command being processed.
@@ -1393,8 +1541,22 @@ class X2goSession(object):
         Clean up X2go session.
 
         """
+        # release terminal session's proxy
         if self.has_terminal_session():
             self.terminal_session.release_proxy()
+
+        # unmount shared folders
+        try:
+            self.unshare_all_local_folders()
+        except X2goSessionException:
+            pass
+
+        # remove client-side session cache
+        if self.terminated and self.has_terminal_session():
+            self.terminal_session.post_terminate_cleanup()
+
+        # destroy terminal session
         if self.has_terminal_session():
             self.terminal_session.__del__()
+
         self.terminal_session = None
