@@ -45,6 +45,8 @@ from x2go.defaults import CURRENT_LOCAL_USER as _CURRENT_LOCAL_USER
 from x2go.defaults import LOCAL_HOME as _LOCAL_HOME
 from x2go.defaults import X2GO_SSH_ROOTDIR as _X2GO_SSH_ROOTDIR
 
+from monkey_patch_paramiko import monkey_patch_paramiko
+monkey_patch_paramiko()
 
 class X2goSSHProxy(paramiko.SSHClient, threading.Thread):
     """\
@@ -151,13 +153,18 @@ class X2goSSHProxy(paramiko.SSHClient, threading.Thread):
             self.remote_host = remote_host
             self.remote_port = int(remote_port)
 
+        # allow more trailing whitespace tolerance in hostnames
+        self.hostname = self.hostname.strip()
+        self.local_host = self.local_host.strip()
+        self.remote_host = self.remote_host.strip()
+
         # enforce IPv4 for localhost addresses!!!
         _hostname = self.hostname
-        if _hostname == 'localhost':
+        if _hostname in ('localhost', 'localhost.localdomain'):
             _hostname = '127.0.0.1'
-        if self.local_host == 'localhost':
+        if self.local_host in ('localhost', 'localhost.localdomain'):
             self.local_host = '127.0.0.1'
-        if self.remote_host == 'localhost':
+        if self.remote_host in ('localhost', 'localhost.localdomain'):
             self.remote_host = '127.0.0.1'
 
         if username is None:
@@ -168,9 +175,10 @@ class X2goSSHProxy(paramiko.SSHClient, threading.Thread):
         self.ssh_rootdir = ssh_rootdir
         paramiko.SSHClient.__init__(self)
 
-        if known_hosts:
-            utils.touch_file(known_hosts)
-            self.load_host_keys(known_hosts)
+        self.known_hosts = known_hosts
+        if self.known_hosts:
+            utils.touch_file(self.known_hosts)
+            self.load_host_keys(self.known_hosts)
 
         if not add_to_known_hosts and session_instance:
             self.set_missing_host_key_policy(checkhosts.X2goInteractiveAddPolicy(caller=self, session_instance=session_instance))
@@ -230,8 +238,16 @@ class X2goSSHProxy(paramiko.SSHClient, threading.Thread):
         Wraps around a Paramiko/SSH host key check.
 
         """
+
+        # hostname rewrite for localhost, force to IPv4
+        _hostname = self.hostname
+
+        # force into IPv4 for localhost connections
+        if _hostname in ('localhost', 'localhost.localdomain'):
+            _hostname = '127.0.0.1'
+
         _valid = False
-        (_valid, _hostname, _port, _fingerprint, _fingerprint_type) = checkhosts.check_ssh_host_key(self, self.hostname, port=self.port)
+        (_valid, _hostname, _port, _fingerprint, _fingerprint_type) = checkhosts.check_ssh_host_key(self, _hostname, port=self.port)
         if not _valid and self.session_instance:
             _valid = self.session_instance.HOOK_check_host_dialog(_hostname, _port, fingerprint=_fingerprint, fingerprint_type=_fingerprint_type)
         return _valid
